/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */

part of "../../flutter_ume_kit_ui_plus.dart";

class WidgetDetailInspector extends StatelessWidget implements Pluggable {
  const WidgetDetailInspector({super.key});

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      theme: ThemeData(primaryColor: Colors.white),
      home: const _DetailPage(),
    );
  }

  @override
  Widget buildWidget(BuildContext? context) => this;

  @override
  ImageProvider<Object> get iconImageProvider =>
      MemoryImage(iconBytesWithDetailInspector);

  @override
  String get name => "WidgetDetail";

  @override
  String get displayName => "WidgetDetail";

  @override
  void onTrigger() {}
}

class _DetailPage extends StatefulWidget {
  const _DetailPage();

  @override
  _DetailPageState createState() => _DetailPageState();
}

class _DetailPageState extends State<_DetailPage> with WidgetsBindingObserver {
  _DetailPageState() : selection = WidgetInspectorService.instance.selection;

  final window = bindingAmbiguate(WidgetsBinding.instance)
      ?.platformDispatcher
      .views
      .first; //window;

  Offset? _lastPointerLocation;

  final InspectorSelection selection;

  void _inspectAt(Offset? position) {
    final List<RenderObject> selected = HitTest.hitTest(position);
    setState(() {
      selection.candidates = selected;
    });
  }

  void _handlePanDown(DragDownDetails event) {
    _lastPointerLocation = event.globalPosition;
    _inspectAt(event.globalPosition);
  }

  void _handleTap() {
    if (_lastPointerLocation != null) {
      _inspectAt(_lastPointerLocation);
    }
    Future.delayed(const Duration(milliseconds: 100), () {
      Navigator.of(context).push(MaterialPageRoute(builder: (ctx) {
        return _InfoPage(
            elements: selection.currentElement!.debugGetDiagnosticChain());
      }));
    });
  }

  @override
  void initState() {
    super.initState();
    selection.clear();
  }

  @override
  Widget build(BuildContext context) {
    List<Widget> children = <Widget>[];
    GestureDetector gesture = GestureDetector(
      onTap: _handleTap,
      onPanDown: _handlePanDown,
      behavior: HitTestBehavior.translucent,
      child: IgnorePointer(
        child: Container(
            width: MediaQuery.of(context).size.width,
            height: MediaQuery.of(context).size.height),
      ),
    );
    children.add(gesture);
    children.add(InspectorOverlay(
        selection: selection, needDescription: false, needEdges: false));
    return Stack(textDirection: TextDirection.ltr, children: children);
  }
}

class _DetailModel {
  List<int> colors = [
    Random().nextInt(256),
    Random().nextInt(256),
    Random().nextInt(256)
  ];
  Element element;

  _DetailModel(this.element);
}

class _InfoPage extends StatefulWidget {
  const _InfoPage({required this.elements});

  final List<Element> elements;

  @override
  __InfoPageState createState() => __InfoPageState();
}

class __InfoPageState extends State<_InfoPage> {
  List<_DetailModel> _showList = <_DetailModel>[];
  late List<_DetailModel> _originalList;

  @override
  void initState() {
    super.initState();
    for (var f in widget.elements) {
      _showList.add(_DetailModel(f));
    }
    _originalList = _showList;
  }

  Widget _dot(List<int> colors) {
    Color randomColor = Color.fromARGB(255, colors[0], colors[1], colors[2]);
    return Container(
      decoration: BoxDecoration(
        shape: BoxShape.circle,
        color: randomColor,
      ),
      width: 10,
      height: 10,
    );
  }

  Widget _cell(_DetailModel model, BuildContext context) {
    return GestureDetector(
      onTap: () {
        Navigator.of(context).push(MaterialPageRoute(builder: (ctx) {
          return Scaffold(
              body: _DetailContent(element: model.element),
              appBar: PreferredSize(
                  preferredSize: const Size.fromHeight(44),
                  child: AppBar(
                      elevation: 0.0, title: const Text("widget_detail"))));
        }));
      },
      child: Container(
        margin: const EdgeInsets.only(top: 6, bottom: 6, left: 12, right: 12),
        child: Row(
          children: [
            Padding(
                padding: const EdgeInsets.only(right: 12),
                child: _dot(model.colors)),
            Expanded(
                child: Text(
              model.element.widget.toStringShort(),
              style: const TextStyle(fontSize: 15),
            ))
          ],
        ),
      ),
    );
  }

  void _textChange(String query) {
    query = query.trim();
    List<_DetailModel> infoList = [];
    for (var model in _originalList) {
      var regExp = RegExp(query, caseSensitive: false);
      if (regExp.hasMatch(model.element.widget.toStringShort())) {
        infoList.add(model);
      }
    }
    setState(() {
      _showList = infoList;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        resizeToAvoidBottomInset: false,
        body: Container(
          child: Column(
            children: <Widget>[
              Padding(
                padding: const EdgeInsets.only(
                    left: 12, right: 12, top: 10, bottom: 10),
                child: SearchBar(
                    placeHolder: "请输入要搜索的widget", onChangeHandle: _textChange),
              ),
              Expanded(
                child: GestureDetector(
                  onPanDown: (_) {
                    FocusScope.of(context).requestFocus(FocusNode());
                  },
                  child: ListView.builder(
                      physics: const BouncingScrollPhysics(),
                      itemBuilder: (_, index) {
                        _DetailModel model = _showList[index];
                        return _cell(model, context);
                      },
                      itemCount: _showList.length),
                ),
              ),
            ],
          ),
        ),
        appBar: PreferredSize(
            preferredSize: const Size.fromHeight(44),
            child: AppBar(
                elevation: 0.0,
                title: RichText(
                  text: TextSpan(
                    text: "widget_build_chain",
                    style: const TextStyle(
                        color: Colors.black,
                        fontSize: 19,
                        fontWeight: FontWeight.w500,
                        fontFamily: "PingFang SC"),
                    children: <TextSpan>[
                      TextSpan(
                          text: "  depth: ${widget.elements.length}",
                          style: const TextStyle(
                              color: Colors.black, fontSize: 11)),
                    ],
                  ),
                ))));
  }
}

class _DetailContent extends StatelessWidget {
  const _DetailContent({required this.element});

  final Element element;

  Widget _titleWidget(String title) {
    return Padding(
      padding: const EdgeInsets.only(top: 12, left: 12),
      child: Text(
        title,
        style: const TextStyle(fontSize: 20, fontWeight: FontWeight.w500),
      ),
    );
  }

  Future<List<String>> getInfo() async {
    Completer<List<String>> completer = Completer();
    String string = element.renderObject!.toStringDeep();
    List<String> list = string.split("\n");
    completer.complete(list);
    return completer.future;
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
        future: getInfo(),
        builder: (_, snapshot) {
          if (snapshot.connectionState == ConnectionState.waiting) {
            return const Center(
                child: CircularProgressIndicator(
                    valueColor: AlwaysStoppedAnimation<Color>(Colors.red)));
          } else if (snapshot.connectionState == ConnectionState.done) {
            return Container(
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  _titleWidget("Widget Description"),
                  Container(
                      constraints: const BoxConstraints(maxHeight: 200),
                      padding:
                          const EdgeInsets.only(top: 12, left: 12, right: 12),
                      child: SingleChildScrollView(
                          child: Text(element.widget.toStringDeep()))),
                  _titleWidget("RenderObject Description"),
                  Expanded(
                      child: Padding(
                          padding: const EdgeInsets.only(
                              top: 12, right: 12, left: 12),
                          child: ListView.builder(
                              physics: const BouncingScrollPhysics(),
                              itemBuilder: (_, index) {
                                return SingleChildScrollView(
                                    scrollDirection: Axis.horizontal,
                                    child: Text((snapshot.data
                                        as List<String>)[index]));
                              },
                              itemCount:
                                  (snapshot.data as List<String>).length))),
                ],
              ),
            );
          } else {
            return Container();
          }
        });
  }
}
